#ifndef BIGNUM_H_DEFINED
#define BIGNUM_H_DEFINED

#include <cctype>
#include <cstdint>
#include <cstddef>
#include <string>
//Possibly to be replaced with map
#include <vector>

//Use actual decimal math, 4 decimal digits to a "word"
class BigNum {
public:
  BigNum(long long=0);
  BigNum(unsigned long long);
  BigNum(long double);
  BigNum(std::string);
  
  //Comparisons: (Others outside class)
  bool operator==(const BigNum& rhs) const;
  bool operator< (const BigNum& rhs) const;
  
  //Arithmetic assignment block: (Non-assignment outside)
  BigNum& operator+=(const BigNum& rhs);
  /*BigNum& operator*=(const BigNum& rhs);
  BigNum& operator/=(const BigNum& rhs);
  BigNum& operator%=(const BigNum& rhs);//*/
  
  //Explicit sign bit is weird
  friend BigNum operator-(BigNum x);
  friend BigNum abs(BigNum x);
  friend bool signbit(const BigNum& x);
  
  //Increasing significance: [0] = ones, [9] = billions
  //Digital access (returns 0-9, 0 for OOB)
  unsigned char operator[](std::size_t);
  //Set digit in position
  BigNum& set(std::size_t);
  ///@IDEA Would it make sense to use << and >> to shift decimal digits?
    //(Efficient *10 and /10 operators mimic closely the binary uses
  
  //For simplicity of implementation:
  //rhs MUST BE < *(this)
  BigNum smallSub(const BigNum& rhs);
private:
  //One greater than actual maximum for clarity
  static constexpr unsigned long long max_word = 10000;
  std::vector<unsigned short> m_vals;
  enum Sign : bool {pos, neg} sign;
  
};

//Apparently literals are only allowed to be unsigned, which makes this easy
inline BigNum operator"" _b (unsigned long long x) {
  return BigNum(x);
}
inline BigNum operator"" _b (long double x) {
  return BigNum(x);
}

//BigNum comparisons
inline bool operator!=(const BigNum& lhs, const BigNum& rhs) {
  return !(lhs == rhs);
}
inline bool operator> (const BigNum& lhs, const BigNum& rhs) {
  return (rhs < lhs);
}
inline bool operator<=(const BigNum& lhs, const BigNum& rhs) {
  return !(rhs < lhs);
}
inline bool operator>=(const BigNum& lhs, const BigNum& rhs) {
  return !(lhs < rhs);
}
//Prevent unnecessary duplication of code
inline BigNum& operator-=(BigNum& lhs, const BigNum& rhs) {
  return lhs+=(-rhs);
}
//Arithmetic block: (inline)
inline BigNum operator+(BigNum lhs, const BigNum& rhs) {
  lhs += rhs;
  return lhs;
}
inline BigNum operator-(const BigNum& lhs, const BigNum& rhs) {
  BigNum tmp;
  if (lhs < rhs) {
    tmp = lhs;
    tmp-=rhs;
  } else {
    tmp = rhs;
    tmp-=lhs;
  }
  return tmp;
}
/*inline BigNum operator*(BigNum lhs, const BigNum& rhs) {
  lhs *= rhs;
  return lhs;
}
inline BigNum operator/(BigNum lhs, const BigNum& rhs) {
  lhs /= rhs;
  return lhs;
}
inline BigNum operator%(BigNum lhs, const BigNum& rhs) {
  lhs %= rhs;
  return lhs;
}//*/
//Prefix increment
inline BigNum& operator++(BigNum& x) {
  return x += 1_b;
}
//prefix decrement
inline BigNum& operator--(BigNum& x) {
  return x += -(1_b);
}
//postfix increment
inline BigNum operator++(BigNum x, int) {
  BigNum tmp(x);
  ++x;
  return tmp;
}
//postfix decrement
inline BigNum operator--(BigNum x, int) {
  BigNum tmp(x);
  --x;
  return tmp;
}
//Unary minus operator
inline BigNum operator-(BigNum x) {
  x.sign = static_cast<BigNum::Sign>(!x.sign);
  return x;
}
//May be useful, simple to implement
inline BigNum abs(BigNum x) {
  x.sign = BigNum::pos;
  return x;
}
inline bool signbit(const BigNum& x) {
  return static_cast<bool>(x.sign);
}

#endif //BIGNUM_H_DEFINED